import json
from io import TextIOWrapper

from edupage_api.exceptions import FailedToUploadFileException
from edupage_api.module import EdupageModule, Module, ModuleHelper


class EduCloudFile:
    def __init__(self, cloud_id: str, extension: str, file_type: str, file: str, name: str):
        self.cloud_id = cloud_id
        self.extension = extension
        self.type = file_type
        self.file = file
        self.name = name

    def get_url(self, edupage: EdupageModule):
        """Get url of given `EduCloudFile`.

        Args:
            edupage (EdupageModule): `Edupage` object.

        Returns:
            str: Direct URL to file.
        """

        return f"https://{edupage.subdomain}.edupage.org{self.file}"

    @staticmethod
    def parse(data: dict):
        """Parse `EduCloudFile` data.

        Args:
            data (dict): Data to parse.

        Returns:
            EduCloudFile: `EduCloudFile` object.
        """

        return EduCloudFile(
            data.get("cloudid"),
            data.get("extension"),
            data.get("type"),
            data.get("file"),
            data.get("name")
        )


class Cloud(Module):
    @ModuleHelper.logged_in
    def upload_file(self, fd: TextIOWrapper) -> EduCloudFile:
        """Upload file to EduPage cloud.

        The file will be hosted forever (and for free) on EduPage's servers. The file is tied to
        your user account, but anybody with a link can view it.

        **Warning!** EduPage limits file size to 50 MB and the file can have only some extensions.
        You can find all supported file extensions on this
        [Edupage help site](https://help.edupage.org/?p=u1/u113/u132/u362/u467).

        If you are willing to upload some files, you will probably have to increase the request
        timeout.

        ```
        >>> with open("image.jpg", "rb") as f:
        ...     edupage.cloud_upload(f)
        <edupage_api.cloud.EduCloudFile object at 0x10b107550>
        ```

        Args:
            fd (TextIOWrapper): File you want to upload.

        Raises:
            FailedToUploadFileException: There was a problem with uploading your file.

        Returns:
            EduCloudFile: `EduCloudFile` object.
        """

        request_url = f"https://{self.edupage.subdomain}.edupage.org/timeline/?akcia=uploadAtt"

        files = {"att": fd}

        response = self.edupage.session.post(request_url, files=files).content.decode()

        try:
            response_json = json.loads(response)
            if response_json.get("status") != "ok":
                raise FailedToUploadFileException("Edupage returned a failing status")

            metadata = response_json.get("data")
            return EduCloudFile.parse(metadata)
        except json.JSONDecodeError:
            raise FailedToUploadFileException("Failed to decode json response")



# For postponed evaluation of annotations
from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Optional

from edupage_api.dbi import DbiHelper
from edupage_api.module import EdupageModule, Module, ModuleHelper


class Gender(Enum):
    MALE = "M"
    FEMALE = "F"

    @staticmethod
    def parse(gender_str: str) -> Optional[Gender]:
        return ModuleHelper.parse_enum(gender_str, Gender)


class EduAccountType(Enum):
    STUDENT = "Student"
    TEACHER = "Teacher"
    PARENT = "Rodic"


class EduAccount:
    def __init__(self, person_id: int, name: str, gender: Gender, in_school_since: datetime, account_type: EduAccountType):
        self.person_id = person_id
        self.name = name
        self.gender = gender
        self.in_school_since = in_school_since
        self.account_type = account_type

    @staticmethod
    def recognize_account_type(person_data: dict) -> EduAccountType:
        if person_data.get("numberinclass") is not None:
            return EduAccountType.STUDENT
        elif person_data.get("classroomid") is not None:
            return EduAccountType.TEACHER
        else:
            return EduAccountType.PARENT

    @staticmethod
    def parse(person_data: dict, person_id: int, edupage: EdupageModule) -> Optional[EduAccount]:
        account_type = EduAccount.recognize_account_type(person_data)

        if account_type == EduAccountType.STUDENT:
            class_id = ModuleHelper.int_or_none(person_data.get("classid"))
            name = DbiHelper(edupage).fetch_student_name(person_id)
            gender = Gender.parse(person_data.get("gender"))
            student_since = ModuleHelper.strptime_or_none(person_data.get("datefrom"), "%Y-%m-%d")
            number_in_class = ModuleHelper.int_or_none(person_data.get("numberinclass"))

            ModuleHelper.assert_none(name)

            return EduStudent(person_id, name, gender, student_since, class_id, number_in_class)
        elif account_type == EduAccountType.TEACHER:
            classroom_id = person_data.get("classroomid")
            classroom_name = DbiHelper(edupage).fetch_classroom_number(
                classroom_id) if ModuleHelper.int_or_none(classroom_id) else ""

            name = DbiHelper(edupage).fetch_teacher_name(person_id)

            gender = Gender.parse(person_data.get("gender"))
            if teacher_since_str := person_data.get("datefrom"):
                teacher_since = datetime.strptime(teacher_since_str, "%Y-%m-%d")
            else:
                teacher_since = None

            if teacher_to_str := person_data.get("dateto"):
                teacher_to = datetime.strptime(teacher_to_str, "%Y-%m-%d")
            else:
                teacher_to = None

            return EduTeacher(person_id, name, gender, teacher_since, classroom_name, teacher_to)
        else:
            return None

    def get_id(self):
        return f"{self.account_type.value}-{self.person_id}"


class EduStudent(EduAccount):
    def __init__(self, person_id: int, name: str, gender: Gender, in_school_since: datetime,
                 class_id: int, number_in_class: int):
        super().__init__(person_id, name, gender, in_school_since, EduAccountType.STUDENT)

        self.class_id = class_id
        self.number_in_class = number_in_class


class EduParent(EduAccount):
    def __init__(self, person_id: int, name: str, gender: Gender, in_school_since: datetime):
        super().__init__(person_id, name, gender, in_school_since, EduAccountType.PARENT)


class EduTeacher(EduAccount):
    def __init__(self, person_id: int, name: str, gender: Gender, in_school_since: datetime,
                 classroom_name: str, teacher_to: datetime):
        super().__init__(person_id, name, gender, in_school_since, EduAccountType.TEACHER)

        self.teacher_to = teacher_to
        self.classroom_name = classroom_name


class People(Module):
    @ModuleHelper.logged_in
    def get_students(self) -> Optional[list]:
        students = DbiHelper(self.edupage).fetch_student_list()
        if students is None:
            return None

        result = []
        for student_id_str in students:
            if not student_id_str:
                continue

            student_id = int(student_id_str)
            student_data = students.get(student_id_str)

            student = EduAccount.parse(student_data, student_id, self.edupage)
            result.append(student)

        return result

    @ModuleHelper.logged_in
    def get_teacher(self, teacher_id: int) -> Optional[EduTeacher]:
        teacher_data = DbiHelper(self.edupage).fetch_teacher_data(teacher_id)
        if teacher_data is None:
            return None

        return EduAccount.parse(teacher_data, teacher_id, self.edupage)

    @ModuleHelper.logged_in
    def get_student(self, student_id: int) -> Optional[EduStudent]:
        student_data = DbiHelper(self.edupage).fetch_student_data(student_id)
        if student_data is None:
            return None

        return EduAccount.parse(student_data, student_id, self.edupage)

    @ModuleHelper.logged_in
    def get_teachers(self) -> Optional[list]:
        teachers = DbiHelper(self.edupage).fetch_teacher_list()
        if teachers is None:
            return None

        result = []
        for teacher_id_str in teachers:
            if not teacher_id_str:
                continue

            teacher_id = int(teacher_id_str)
            teacher_data = teachers.get(teacher_id_str)

            teacher = EduAccount.parse(teacher_data, teacher_id, self.edupage)
            result.append(teacher)

        return result



import json
from datetime import datetime
from typing import List, Optional, Union

from edupage_api.dbi import DbiHelper
from edupage_api.exceptions import MissingDataException
from edupage_api.module import EdupageModule, Module, ModuleHelper
from edupage_api.people import EduTeacher, People


class Lesson:
    def __init__(self, teachers: List[EduTeacher], classrooms: List[str],
                 start_of_lesson: datetime, end_of_lesson: datetime,
                 online_lesson_link: Optional[str], subject_id: int, name: str):
        self.teachers = teachers
        self.classrooms = classrooms
        self.start_of_lesson = start_of_lesson
        self.end_of_lesson = end_of_lesson
        self.online_lesson_link = online_lesson_link
        self.subject_id = subject_id
        self.name = name

    def is_online_lesson(self) -> bool:
        return self.online_lesson_link is None

    @ModuleHelper.online_lesson
    def sign_into_lesson(self, edupage: EdupageModule):
        request_url = f"https://{edupage.subdomain}.edupage.org/dashboard/eb.php"

        response = edupage.session.get(request_url)

        gse_hash = response.content.decode() \
                                   .split("gsechash=")[1] \
                                   .split('"')[1]

        request_url = f"https://{edupage.subdomain}.edupage.org/dashboard/server/onlinelesson.js?__func=getOnlineLessonOpenUrl"
        today = datetime.today()
        post_data = {
            "__args": [
                None,
                {
                    "click": True,
                    "date": today.strftime("%Y-%m-%d"),
                    "ol_url": self.online_lesson_link,
                    "subjectid": self.subject_id
                }
            ],
            "__gsh": gse_hash
        }

        response = edupage.session.post(request_url, json=post_data)
        return json.loads(response.content.decode()).get("reload") is not None


class Timetable:
    def __init__(self, lessons: List[Lesson]):
        self.lessons = lessons

    def __iter__(self):
        return iter(self.lessons)

    def get_lesson_at_time(self, time: datetime):
        # this is done to drop the date part of the datetime
        hour, minute = time.hour, time.minute
        time = datetime(hour=hour, minute=minute)

        for lesson in self.lessons:
            if time >= lesson.start_of_lesson and time <= lesson.end_of_lesson:
                return lesson

    def get_next_lesson_at_time(self, time: datetime):
        # this is done to drop the date part of the datetime
        hour, minute = time.hour, time.minute
        time = datetime(hour=hour, minute=minute)

        for lesson in self.lessons:
            if time < lesson.start_of_lesson:
                return lesson

    def get_next_online_lesson_at_time(self, time: datetime):
        for lesson in self.lessons:
            if time < lesson.start_of_lesson and lesson.is_online_lesson():
                return lesson

    def get_first_lesson(self):
        if len(self.lessons) > 0:
            return self.lessons[0]

    def get_last_lesson(self):
        if len(self.lessons) > 0:
            return self.lessons[-1]


class Timetables(Module):
    def __get_dp(self) -> dict:
        dp = self.edupage.data.get("dp")
        if dp is None:
            raise MissingDataException()

        return dp

    def get_timetable(self, date: datetime) -> Union[Timetable, None]:
        dp = self.__get_dp()

        dates = dp.get("dates")
        date_plans = dates.get(date.strftime("%Y-%m-%d"))
        if date_plans is None:
            raise MissingDataException()

        plan = date_plans.get("plan")

        lessons = []
        for subject in plan:
            header = subject.get("header")
            if len(header) == 0:
                continue

            subject_id = int(subject.get("subjectid"))
            if subject_id:
                subject_name = DbiHelper(self.edupage).fetch_subject_name(subject_id)
            else:
                subject_name = header[0].get("text")

            teachers = []
            teacher_ids = subject.get("teacherids")
            if teacher_ids is not None and len(teacher_ids) != 0:
                for teacher_id_str in teacher_ids:
                    if not teacher_id_str:
                        continue

                    teacher_id = int(teacher_id_str)
                    teacher = People(self.edupage).get_teacher(teacher_id)

                    teachers.append(teacher)

            classrooms = []
            classroom_ids = subject.get("classroomids")
            if classroom_ids is not None and len(classroom_ids) != 0:
                for classroom_id_str in classroom_ids:
                    if not classroom_id_str:
                        continue

                    classroom_id = int(classroom_id_str)
                    classroom_number = DbiHelper(self.edupage).fetch_classroom_number(classroom_id)

                    classrooms.append(classroom_number)

            start_of_lesson_str = subject.get("starttime")
            end_of_lesson_str = subject.get("endtime")

            ModuleHelper.assert_none(start_of_lesson_str, end_of_lesson_str)

            start_of_lesson = datetime.strptime(start_of_lesson_str, "%H:%M")
            end_of_lesson = datetime.strptime(end_of_lesson_str, "%H:%M")

            online_lesson_link = subject.get("ol_url")

            lesson = Lesson(teachers, classrooms, start_of_lesson, end_of_lesson,
                            online_lesson_link, subject_id, subject_name)
            lessons.append(lesson)

        return Timetable(lessons)



import json
from datetime import datetime

from edupage_api.dbi import DbiHelper
from edupage_api.exceptions import FailedToParseGradeDataError
from edupage_api.module import Module, ModuleHelper
from edupage_api.people import EduTeacher


class EduGrade:
    def __init__(self, event_id: int, title: str, grade_n: int,
                 date: datetime, subject_id: int, subject_name: str,
                 teacher: EduTeacher, max_points: float, importance: float,
                 verbal: True, percent: float):
        self.event_id = event_id
        self.title = title
        self.grade_n = grade_n
        self.date = date
        self.subject_id = subject_id
        self.subject_name = subject_name
        self.teacher = teacher
        self.max_points = max_points
        self.importance = importance
        self.verbal = verbal
        self.percent = percent


class Grades(Module):
    def __parse_grade_data(self, data: str) -> dict:
        json_string = data.split(".znamkyStudentViewer(")[1] \
                          .split(");\r\n\t\t});\r\n\t\t</script>")[0]

        return json.loads(json_string)

    def __get_grade_data(self):
        request_url = f"https://{self.edupage.subdomain}.edupage.org/znamky/"

        response = self.edupage.session.get(request_url).content.decode()

        try:
            return self.__parse_grade_data(response)
        except json.JSONDecodeError:
            raise FailedToParseGradeDataError("Failed to parse JSON")

    @ModuleHelper.logged_in
    def get_grades(self) -> list[EduGrade]:
        grade_data = self.__get_grade_data()

        grades = grade_data.get("vsetkyZnamky")
        grade_details = grade_data.get("vsetkyUdalosti").get("edupage")

        output = []
        for grade in grades:
            event_id_str = grade.get("udalostid")
            if not event_id_str:
                continue

            event_id = int(event_id_str)

            details = grade_details.get(event_id_str)
            title = details.get("p_meno")

            grade_n = ModuleHelper.parse_int(grade.get("data"))

            date_str = grade.get("datum")
            date = datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")

            subject_id_str = details.get("PredmetID")
            if subject_id_str is None or subject_id_str == "vsetky":
                continue

            subject_id = int(subject_id_str)
            subject_name = DbiHelper(self.edupage).fetch_subject_name(subject_id)

            teacher_id_str = details.get("UcitelID")
            if teacher_id_str is None:
                teacher = None
            else:
                teacher_id = int(teacher_id_str)
                teacher_data = DbiHelper(self.edupage).fetch_teacher_data(teacher_id)

                teacher = EduTeacher.parse(teacher_data, teacher_id, self.edupage)

            max_points = details.get("p_vaha_body")
            max_points = int(max_points) if max_points is not None else None

            importance = details.get("p_vaha")
            importance = 0 if float(importance) == 0 else 20 / float(importance)

            try:
                verbal = False

                if max_points:
                    percent = round(float(grade_n) / float(max_points) * 100, 2)
                else:
                    percent = None
            except:
                verbal = True

            grade = EduGrade(event_id, title, grade_n, date, subject_id,
                             subject_name, teacher, max_points, importance, verbal, percent)
            output.append(grade)

        return output



from typing import Optional

from edupage_api.module import Module, ModuleHelper


class DbiHelper(Module):
    def __get_dbi(self) -> dict:
        return self.edupage.data.get("dbi")

    def __get_item_group(self, item_group_name: str) -> Optional[dict]:
        dbi = self.__get_dbi()
        if dbi is None:
            return None

        return dbi.get(item_group_name)

    def __get_item_with_id(self, item_group_name: str, item_id: int) -> Optional[dict]:
        items_in_group = self.__get_item_group(item_group_name)
        if items_in_group is None:
            return None

        return items_in_group.get(str(item_id))

    def __get_full_name(self, person_item: dict) -> str:
        first_name = person_item.get("firstname")
        last_name = person_item.get("lastname")

        return f"{first_name} {last_name}"

    def fetch_subject_name(self, subject_id: int) -> Optional[str]:
        subject_item = self.__get_item_with_id("subjects", subject_id)
        if subject_item is not None:
            return subject_item.get("short")

    def fetch_classroom_number(self, classroom_id: int) -> Optional[str]:
        classroom_item = self.__get_item_with_id("classrooms", classroom_id)
        if classroom_item is not None:
            return classroom_item.get("short")

    def fetch_class_name(self, class_id: int) -> Optional[str]:
        class_item = self.__get_item_with_id("classes", class_id)
        if class_item is not None:
            return class_item.get("short")

    def fetch_teacher_name(self, teacher_id: int) -> Optional[str]:
        teacher_item = self.__get_item_with_id("teachers", teacher_id)
        if teacher_item is not None:
            return self.__get_full_name(teacher_item)

    def fetch_student_name(self, student_id: int) -> Optional[str]:
        student_item = self.__get_item_with_id("students", student_id)
        if student_item is not None:
            return self.__get_full_name(student_item)

    def fetch_student_list(self) -> Optional[list]:
        return self.__get_item_group("students")

    def fetch_teacher_list(self) -> Optional[list]:
        return self.__get_item_group("teachers")

    def fetch_teacher_data(self, teacher_id: int) -> Optional[dict]:
        return self.__get_item_with_id("teachers", teacher_id)

    def fetch_student_data(self, student_id: int) -> Optional[dict]:
        return self.__get_item_with_id("students", student_id)

    def fetch_student_data_by_name(self, student_name: str) -> Optional[dict]:
        item_group = self.__get_item_group("students")

        for student_id in item_group:
            student_data = item_group.get(student_id)
            if self.__get_full_name(student_data) in student_name:
                student_data["id"] = student_id
                return student_data

    def fetch_teacher_data_by_name(self, teacher_name: str) -> Optional[dict]:
        item_group = self.__get_item_group("teachers")

        for teacher_id in item_group:
            teacher_data = item_group.get(teacher_id)
            if self.__get_full_name(teacher_data) in teacher_name:
                teacher_data["id"] = teacher_id
                return teacher_data

    def fetch_parent_data_by_name(self, parent_name: str) -> Optional[dict]:
        item_group = self.__get_item_group("parents")

        for parent_id in item_group:
            parent_data = item_group.get(parent_id)
            if self.__get_full_name(parent_data) in parent_name:
                parent_data["id"] = parent_id
                return parent_data

    def fetch_person_data_by_name(self, name: str) -> Optional[dict]:
        teacher_data = self.fetch_teacher_data_by_name(name)
        student_data = self.fetch_student_data_by_name(name)
        parent_data = self.fetch_parent_data_by_name(name)

        return ModuleHelper.return_first_not_null(teacher_data, student_data, parent_data)



# For postponed evaluation of annotations
from __future__ import annotations

import json
from datetime import date
from enum import Enum
from typing import Optional, Union

from edupage_api.exceptions import ExpiredSessionException, InvalidTeacherException
from edupage_api.module import Module, ModuleHelper
from edupage_api.people import EduTeacher, People


class Action(Enum):
    ADDITION = "add"
    CHANGE = "change"
    DELETION = "remove"

    @staticmethod
    def parse(string: str) -> Optional[Action]:
        return ModuleHelper.parse_enum(string, Action)


class TimetableChange:
    def __init__(self, change_class: str, lesson_n: int, title: str, action: Union[Action, tuple[int, int]]):
        self.change_class = change_class
        self.lesson_n = lesson_n
        self.title = title
        self.action = action


class Substitution(Module):
    def __get_substitution_data(self, date: date) -> str:
        url = (f"https://{self.edupage.subdomain}.edupage.org/substitution/server/viewer.js"
               "?__func=getSubstViewerDayDataHtml")

        data = {
            "__args": [None, {
                "date": date.strftime("%Y-%m-%d"),
                "mode": "classes"
            }],
            "__gsh": self.edupage.gsec_hash
        }

        response = self.edupage.session.post(url, json=data).content.decode()
        response = json.loads(response)

        if response.get("reload"):
            raise ExpiredSessionException("Invalid gsec hash! "
                                          "(Expired session, try logging in again!)")

        return response.get("r")

    @ModuleHelper.logged_in
    def get_missing_teachers(self, date: date) -> list[EduTeacher]:
        html = self.__get_substitution_data(date)
        missing_teachers_string = (html.split("<span class=\"print-font-resizable\">")[1]
                                       .split("</span>")[0])

        _title, missing_teachers = missing_teachers_string.split(": ")

        all_teachers = People(self.edupage).get_teachers()

        missing_teachers = [
            (t.strip()
              .split("  (")[0])
            for t in missing_teachers.split(", ")
        ]

        try:
            missing_teachers = [
                list(filter(lambda x: x.name == t, all_teachers))[0]
                for t in missing_teachers
            ]
        except IndexError:
            raise InvalidTeacherException("Invalid teacher in substitution! "
                                          "(The teacher is no longer frequenting this school)")

        return missing_teachers

    @ModuleHelper.logged_in
    def get_timetable_changes(self, date: date) -> list[TimetableChange]:
        html = self.__get_substitution_data(date)

        class_delim = ("</div><div class=\"section print-nobreak\">"
                       "<div class=\"header\"><span class=\"print-font-resizable\">")
        changes_by_class_dirty = html.split(class_delim)[1:]

        footer_delim = ("<div style=\"text-align:center;font-size:12px\">"
                        "<a href=\"https://www.asctimetables.com\" target=\"_blank\">"
                        "www.asctimetables.com</a> -")
        changes_by_class_dirty[-1] = changes_by_class_dirty[-1].split(footer_delim)[0]

        changes = [
            (x.replace("</div>", "")
              .replace("<div class=\"period\">", "")
              .replace("<span class=\"print-font-resizable\">", "")
              .replace("<div class=\"info\">", ""))
            for x in changes_by_class_dirty
        ]

        lesson_changes = []
        for class_changes in changes:
            class_changes_data = class_changes.split("</span><div class=\"rows\">")
            change_class = class_changes_data[0]

            class_changes_rows = class_changes_data[1].split("<div class=\"row ")[1:]

            for change in class_changes_rows:
                change = change.replace("\">", "</span>")
                action, lesson_n, title = change.split("</span>")[:-1]

                if "<img src=" in title:
                    title = title.split(">")[1]

                action = Action.parse(action)

                if "-" in lesson_n:
                    lesson_from, lesson_to = lesson_n.split(" - ")
                    lesson_n = (ModuleHelper.parse_int(lesson_from),
                                ModuleHelper.parse_int(lesson_to))
                else:
                    lesson_n = ModuleHelper.parse_int(lesson_n)

                lesson_change = TimetableChange(change_class, lesson_n, title, action)
                lesson_changes.append(lesson_change)

        return lesson_changes



from typing import Union

from edupage_api.module import Module, ModuleHelper
from edupage_api.people import EduAccount


class Messages(Module):
    def send_message(self, recipients: Union[list[EduAccount], EduAccount], body: str):
        recipient_string = ""

        if isinstance(recipients, list):
            for i, recipient in recipients:
                recipient_string += f"{recipient.get_id()}"
                if i == len(recipients) - 1:
                    recipient_string += ";"
        else:
            recipient_string = recipients.get_id()

        data = {
            "receipt": "0",
            "selectedUser": recipient_string,
            "text": body,
            "typ": "sprava",
            "attachments": "{}"
        }

        params = (
            ("akcia", "createItem"),
        )

        request_url = f"https://{self.edupage.subdomain}.edupage.org/timeline"
        self.edupage.session.post(request_url, params=params,
                                  data=ModuleHelper.encode_form_data(data))



import json
import urllib.parse


class GradeUtil:
    def __init__(self, grade_data):
        self.data = grade_data

    def id_to_teacher(self, teacher_id):
        if teacher_id is None:
            return None

        try:
            teachers = self.data.get("ucitelia")

            # This dict contains data about the
            # date when this teacher was employed
            # or when will he/she retire / is planned quit this job
            # (datefrom, dateto)
            teacher = teachers.get(teacher_id)

            teacher_name = teacher.get("firstname") + " " + teacher.get("lastname")
        except:
            teacher_name = None

        return teacher_name


class LessonUtil:
    @staticmethod
    def is_online_lesson(lesson):
        # We cannot import from timetables.py because of circular import
        return "EduOnlineLesson" in str(type(lesson))


class IdUtil:
    def __init__(self, data):
        self.data = data
        self.dbi = data.get("dbi")

    def id_to_class(self, c_id):
        if c_id is None:
            return None

        try:
            class_name = self.dbi.get("classes").get(c_id).get("name")
        except:
            class_name = None

        return class_name

    def id_to_teacher(self, t_id):
        if t_id is None:
            return None

        try:
            teacher_data = self.dbi.get("teachers").get(t_id)
            teacher_full_name = teacher_data.get(
                "firstname") + " " + teacher_data.get("lastname")
        except:
            teacher_full_name = None

        return teacher_full_name

    def id_to_classroom(self, c_id):
        if c_id is None:
            return None

        try:
            classroom = self.dbi.get("classrooms").get(c_id).get("short")
        except:
            classroom = None

        return classroom

    def id_to_subject(self, s_id):
        if s_id is None:
            return s_id

        try:
            subject = self.dbi.get("subjects").get(s_id).get("short")
        except:
            subject = None

        return subject


class RequestUtil:
    @staticmethod
    def urlencode(string):
        return urllib.parse.quote(string)

    @staticmethod
    def encode_form_data(data):
        output = ""
        for i, key in enumerate(data.keys(), start=0):
            value = data[key]
            entry = f"{RequestUtil.urlencode(key)}={RequestUtil.urlencode(value)}"

            output += f"&{entry}" if i != 0 else entry
        return output

    @staticmethod
    def encode_attachments(attachments):
        output = {attachment.url: attachment.filename for attachment in attachments}

        return json.dumps(output)



from requests import Response

from edupage_api.module import Module


class CustomRequest(Module):
    def custom_request(self, url: str, method: str, data: str = "", headers: dict = {}) -> Response:
        if method == "GET":
            response = self.edupage.session.get(url, headers=headers)
        elif method == "POST":
            response = self.edupage.session.post(url, data=data, headers=headers)

        return response



import urllib.parse
from datetime import datetime
from enum import Enum
from functools import wraps
from typing import Union

import requests

from edupage_api.exceptions import (MissingDataException,
                                    NotAnOnlineLessonError,
                                    NotLoggedInException)


class EdupageModule:
    subdomain: str
    session: requests.Session
    data: dict
    is_logged_in: bool
    gsec_hash: str


class Module:
    def __init__(self, edupage: EdupageModule):
        self.edupage = edupage


class ModuleHelper:
    # Helper Functions

    @staticmethod
    def int_or_none(val: str) -> Union[int, None]:
        if val.isdigit():
            return int(val)

    @staticmethod
    def parse_int(val: str) -> int:
        try:
            return int("".join(filter(str.isdigit, val)))
        except ValueError:
            return None

    """
    If any argument of this function is none, it throws MissingDataException
    """
    @staticmethod
    def assert_none(*args):
        if None in args:
            raise MissingDataException()

    @staticmethod
    def parse_enum(string: str, enum_type: Enum):
        filtered = list(filter(lambda x: x.value == string, list(enum_type)))

        if not filtered:
            return None

        return filtered[0]

    @staticmethod
    def return_first_not_null(*args):
        for x in args:
            if x:
                return x

    @staticmethod
    def urlencode(string: str) -> str:
        return urllib.parse.quote(string)

    @staticmethod
    def encode_form_data(data: dict) -> str:
        output = ""
        for i, key in enumerate(data.keys(), start=0):
            value = data[key]
            entry = f"{ModuleHelper.urlencode(key)}={ModuleHelper.urlencode(value)}"

            output += f"&{entry}" if i != 0 else entry
        return output

    @staticmethod
    def strptime_or_none(date_string: str, format: str) -> Union[datetime, None]:
        try:
            return datetime.strptime(date_string, format)
        except ValueError:
            return None

    # Decorators

    """
    Throws NotLoggedInException if someone uses a method with this decorator
    and hasn't logged in yet
    """
    @staticmethod
    def logged_in(method):
        @wraps(method)
        def __impl(self: Module, *method_args, **method_kwargs):
            if not self.edupage.is_logged_in:
                raise NotLoggedInException()

            return method(self, *method_args, **method_kwargs)
        return __impl

    @staticmethod
    def online_lesson(method):
        @wraps(method)
        def __impl(self, *method_args, **method_kwargs):
            if self.online_lesson_link is None:
                raise NotAnOnlineLessonError()
            return method(self, *method_args, **method_kwargs)

        return __impl



class NotLoggedInException(Exception):
    pass


class MissingDataException(Exception):
    pass


class BadCredentialsException(Exception):
    pass


class NotAnOnlineLessonError(Exception):
    pass


class FailedToRateException(Exception):
    pass


class FailedToChangeLunchError(Exception):
    pass


class FailedToUploadFileException(Exception):
    pass


class FailedToParseGradeDataError(Exception):
    pass


class ExpiredSessionException(Exception):
    pass


class InvalidTeacherException(Exception):
    pass



# For postponed evaluation of annotations
from __future__ import annotations

import json
from datetime import datetime
from enum import Enum
from typing import Optional

from edupage_api.dbi import DbiHelper
from edupage_api.module import Module, ModuleHelper
from edupage_api.people import EduAccount


# data.dbi.event_types
class EventType(Enum):
    # Messages
    MESSAGE = "sprava"
    NEWS = "news"

    # ****************************************

    # Exam types
    BIG_EXAM = "bexam"
    HOMEWORK = "homework"
    ORAL_EXAM = "oexam"
    PAPER = "rexam"
    PROJECT_EXAM = "pexam"
    SHORT_EXAM = "sexam"
    TESTING = "testing"

    # Exam manipulation
    EXAM_ASSIGNMENT = "testpridelenie"
    EXAM_EVALUATION = "testvysledok"
    HOMEWORK_STUDENT_STATE = "homeworkstudentstav"
    HOMEWORK_TEST = "etesthw"
    TEST_RESULT = "testvysledok"

    # ****************************************

    # Grades
    GRADE = "znamka"
    GRADES_DOC = "znamkydoc"

    # ****************************************

    # Events
    CLASS_BOOK = "other_cb"
    CLASS_TEACHER_EVENT = "ctevent"
    CLASSIFICATION_MEETING = "bmeeting"
    CULTURE = "culture"
    EVENT = "event"
    EXCURSION = "excursion"
    PARENTS_EVENING = "parentsevening"
    PROCESS = "process"
    SCHOOL_EVENT = "schoolevent"
    SCHOOL_TRIP = "trip"
    TEACHER_MEETING = "meeting"

    # Free days
    FREE_DAY = "freeday"
    HOLIDAY = "holiday"
    SHORT_HOLIDAY = "sholiday"
    TT_CANCEL = "ttcancel"

    # Lessons
    CLASS_TEACHER_LESSON = "ctlesson"
    DISTANT_LEARNING = "distant"
    LESSON = "lesson"
    PROJECT = "project"
    PROJECT_LESSON = "plesson"
    SAFETY_INSTRUCTIONING = "other_safety"
    TUTORING = "rlesson"

    # ****************************************

    # Timetable
    TIMETABLE = "timetable"

    # Substitution
    BOOKED_ROOM = "bookroom"
    CHANGE_ROOM = "changeroom"
    SUBSTITUTION = "substitution"

    # ****************************************

    # Presence
    ARRIVAL_TO_SCHOOL = "pipnutie"

    # Absence
    EXCUSED_LESSON = "ospravedlnenka"
    REPRESENTATION = "representation"
    STUDENT_ABSENT = "student_absent"

    # ****************************************

    # Food
    FOOD_CREDIT = "strava_kredit"
    FOOD_SERVED = "strava_vydaj"
    NEW_MENU = "h_stravamenu"

    # ****************************************

    # Contest
    CONFIRMATION = "confirmation"
    CONTEST = "contest"

    # Photo album
    ALBUM = "album"

    # Other
    OTHER = "other"

    # Helper
    H_ATTENDANCE = "h_attendance"
    H_BEE = "h_vcelicka"
    H_CLEARCACHE = "h_clearcache"
    H_CLEARDBI = "h_cleardbi"
    H_CLEARISICDATA = "h_clearisicdata"
    H_CLEARPLANS = "h_clearplany"
    H_CONTENST = "h_contest"
    H_DAILYPLAN = "h_dailyplan"
    H_EDUSETTINGS = "h_edusettings"
    H_FINANCES = "h_financie"
    H_GRADES = "h_znamky"
    H_HOMEWORK = "h_homework"
    H_PROCESS = "h_process"
    H_PROCESSTYPES = "h_processtypes"
    H_SETTINGS = "h_settings"
    H_SUBSTITUTION = "h_substitution"
    H_TIMETABLE = "h_timetable"

    @staticmethod
    def parse(event_type_str: str) -> Optional[EventType]:
        return ModuleHelper.parse_enum(event_type_str, EventType)


class TimelineEvent:
    def __init__(self, event_id: int, timestamp: datetime, text: str,
                 author: EduAccount, recipient: EduAccount, event_type: EventType,
                 additional_data: dict):
        self.event_id = event_id
        self.timestamp = timestamp
        self.text = text
        self.author = author
        self.recipient = recipient
        self.event_type = event_type
        self.additional_data = additional_data


class TimelineEvents(Module):
    @ModuleHelper.logged_in
    def get_notifications(self):
        output = []

        timeline_items = self.edupage.data.get("items")

        for event in timeline_items:
            event_id_str = event.get("timelineid")
            if not event_id_str:
                continue

            event_id = int(event_id_str)
            event_data = json.loads(event.get("data"))

            event_type_str = event.get("typ")
            if not event_id_str:
                continue
            event_type = EventType.parse(event_type_str)

            if event_type is None:
                print(event_type_str)

            event_timestamp = datetime.strptime(event.get("timestamp"), "%Y-%m-%d %H:%M:%S")
            text = event.get("text")

            # what about different languages?
            # for message event type
            if text.startswith("Dôležitá správa"):
                text = event_data.get("messageContent")

            if text == "":
                try:
                    text = event_data.get("nazov")
                except:
                    text = ""

            # todo: add support for "*"
            recipient_name = event.get("user_meno")
            recipient_data = DbiHelper(self.edupage).fetch_person_data_by_name(recipient_name)

            if recipient_name in ["*", "Celá škola"]:
                recipient = "*"
            elif type(recipient_name) == str:
                recipient = recipient_name
            else:
                ModuleHelper.assert_none(recipient_data)

                recipient = EduAccount.parse(recipient_data, recipient_data.get("id"), self.edupage)

            # todo: add support for "*"
            author_name = event.get("vlastnik_meno")
            author_data = DbiHelper(self.edupage).fetch_person_data_by_name(author_name)

            if author_name == "*":
                author = "*"
            elif type(author_name) == str:
                author = author_name
            else:
                ModuleHelper.assert_none(author_data)
                author = EduAccount.parse(author_data, author_data.get("id"), self.edupage)

            additional_data = event.get("data")
            if additional_data and type(additional_data) == str:
                additional_data = json.loads(additional_data)

            event = TimelineEvent(event_id, event_timestamp, text, author,
                                  recipient, event_type, additional_data)
            output.append(event)
        return output



import functools
from datetime import date, datetime
from io import TextIOWrapper
from typing import Optional

import requests
from requests import Response

from edupage_api.cloud import Cloud, EduCloudFile
from edupage_api.custom_request import CustomRequest
from edupage_api.grades import EduGrade, Grades
from edupage_api.login import Login
from edupage_api.lunches import Lunch, Lunches
from edupage_api.messages import Messages
from edupage_api.module import EdupageModule
from edupage_api.people import EduAccount, EduStudent, EduTeacher, People
from edupage_api.substitution import Substitution, TimetableChange
from edupage_api.timeline import TimelineEvent, TimelineEvents
from edupage_api.timetables import Timetable, Timetables


class Edupage(EdupageModule):
    def __init__(self, request_timeout=5):
        """Initialize `Edupage` object.

        Args:
            request_timeout (int, optional): Length of request timeout in seconds.
                If want to upload bigger files, you will have to increase its value.
                Defaults to `5`.
        """

        self.data = None
        self.is_logged_in = False
        self.subdomain = None
        self.gsec_hash = None

        self.session = requests.session()
        self.session.request = functools.partial(self.session.request, timeout=request_timeout)

    def login(self, username: str, password: str, subdomain: str):
        """Login while specifying the subdomain to log into.

        Args:
            username (str): Your username.
            password (str): Your password.
            subdomain (str): Subdomain of your school (https://{subdomain}.edupage.org).
        """

        Login(self).login(username, password, subdomain)

    def login_auto(self, username: str, password: str):
        """Login using https://portal.edupage.org. If this doesn't work, please use `Edupage.login`.

        Args:
            username (str): Your username.
            password (str): Your password.
        """

        Login(self).login_auto(username, password)

    def get_students(self) -> Optional[list[EduStudent]]:
        """Get list of all students in your class.

        Returns:
            Optional[list[EduStudent]]: List of `EduStudent`s.
        """

        return People(self).get_students()

    def get_teachers(self) -> Optional[list[EduTeacher]]:
        """Get list of all teachers in your school.

        Returns:
            Optional[list[EduTeacher]]: List of `EduTeacher`s.
        """

        return People(self).get_teachers()

    def send_message(self, recipients: Optional[list[EduAccount]], body: str):
        """Send message.

        Args:
            recipients (Optional[list[EduAccount]]): Recipients of your message (list of `EduAccount`s).
            body (str): Body of your message.
        """

        Messages(self).send_message(recipients, body)

    def get_timetable(self, date: datetime) -> Optional[Timetable]:
        """Get timetable.

        Args:
            date (datetime): Date from which you want to get timetable.

        Returns:
            Optional[Timetable]: Timetable object for entered date.
        """

        return Timetables(self).get_timetable(date)

    def get_lunches(self, date: datetime) -> Optional[Lunch]:
        """Get lunches.

        Args:
            date (datetime): Date from which you want to get lunches.

        Returns:
            Optional[Lunch]: Lunch object for entered date.
        """

        return Lunches(self).get_lunch(date)

    def get_notifications(self) -> list[TimelineEvent]:
        """Get list of all available notifications.

        Returns:
            list[TimelineEvent]: List of `TimelineEvent`s.
        """

        return TimelineEvents(self).get_notifications()

    def cloud_upload(self, fd: TextIOWrapper) -> EduCloudFile:
        """Upload file to EduPage cloud.

        Args:
            fd (TextIOWrapper): File you want to upload.

        Returns:
            EduCloudFile: Object of uploaded file.
        """

        return Cloud(self).upload_file(fd)

    def get_grades(self) -> list[EduGrade]:
        """Get list of all available grades.

        Returns:
            list[EduGrade]: List of `EduGrade`s.
        """

        return Grades(self).get_grades()

    def get_user_id(self) -> str:
        """Get your EduPage user ID.

        Returns:
            str: Your EduPage user ID.
        """

        return self.data.get("userid")

    def custom_request(self, url: str, method: str, data: str = "", headers: dict = {}) -> Response:
        """Send custom request to EduPage.

        Args:
            url (str): URL endpoint.
            method (str): Method (`GET` or `POST`).
            data (str, optional): Request data. Defaults to `""`.
            headers (dict, optional): Request headers. Defaults to `{}`.

        Returns:
            Response: Response.
        """

        return CustomRequest(self).custom_request(url, method, data, headers)

    def get_missing_teachers(self, date: date) -> list[EduTeacher]:
        """Get missing teachers for a given date.

        Args:
            date (datetime.date): The date you want to get this information for.

        Returns:
            list[EduTeacher]: List of the missing teachers for `date`.
        """
        return Substitution(self).get_missing_teachers(date)

    def get_timetable_changes(self, date: date) -> list[TimetableChange]:
        """Get the changes in the timetable for a given date.

        Args:
            date (datetime.date): The date you want to get this information for.

        Returns:
            list[TimetableChange]: List of changes in the timetable
        """
        return Substitution(self).get_timetable_changes(date)



import json

from edupage_api.exceptions import BadCredentialsException
from edupage_api.module import Module


class Login(Module):
    def __parse_login_data(self, data):
        json_string = (data.split("userhome(", 1)[1]
                           .rsplit(");", 2)[0]
                           .replace("\t", "")
                           .replace("\n", "")
                           .replace("\r", ""))

        self.edupage.data = json.loads(json_string)
        self.edupage.is_logged_in = True

        self.edupage.gsec_hash = data.split("ASC.gsechash=\"")[1].split("\"")[0]

    def login_auto(self, username: str, password: str):
        """Login using https://portal.edupage.org. If this doesn't work, please use `Login.login`.

        Args:
            username (str): Your username.
            password (str): Your password.

        Raises:
            BadCredentialsException: Your credentials are invalid.
        """

        parameters = {
            "meno": username,
            "heslo": password,
            "akcia": "login"
        }

        request_url = "https://portal.edupage.org/index.php?jwid=jw2&module=login"
        response = self.edupage.session.post(request_url, params=parameters)

        if "wrongPassword" in response.url:
            raise BadCredentialsException()

        data = response.content.decode()

        self.__parse_login_data(data)
        self.edupage.subdomain = data.split("-->")[0].split(" ")[-1]

    def login(self, username: str, password: str, subdomain: str):
        """Login while specifying the subdomain to log into.

        Args:
            username (str): Your username.
            password (str): Your password.
            subdomain (str): Subdomain of your school (https://{subdomain}.edupage.org).

        Raises:
            BadCredentialsException: Your credentials are invalid.
        """

        request_url = f"https://{subdomain}.edupage.org/login/index.php"

        csrf_response = self.edupage.session.get(request_url).content.decode()

        csrf_token = csrf_response.split("name=\"csrfauth\" value=\"")[1].split("\"")[0]

        parameters = {
            "username": username,
            "password": password,
            "csrfauth": csrf_token
        }

        request_url = f"https://{subdomain}.edupage.org/login/edubarLogin.php"

        response = self.edupage.session.post(request_url, parameters)

        if "bad=1" in response.url:
            raise BadCredentialsException()

        self.__parse_login_data(response.content.decode())
        self.edupage.subdomain = subdomain



import json
from datetime import datetime
from typing import List, Optional

from edupage_api.exceptions import (FailedToChangeLunchError,
                                    FailedToRateException,
                                    NotLoggedInException)
from edupage_api.module import EdupageModule, Module, ModuleHelper


class Rating:
    def __init__(self, mysql_date, boarder_id,
                 quality_average, quantity_average,
                 quality_ratings, quantity_ratings):
        self.quality_ratings = quality_ratings
        self.quality_average = quality_average

        self.quantity_ratings = quantity_ratings
        self.quantity_average = quantity_average

        self.__date = mysql_date
        self.__boarder_id = boarder_id

    def rate(self, edupage: EdupageModule, quantity: int, quality: int):
        if not edupage.is_logged_in:
            raise NotLoggedInException()

        request_url = f"https://{edupage.subdomain}.edupage.org/menu/"

        data = {
            "akcia": "ulozHodnotenia",
            "stravnikid": self.__boarder_id,
            "mysqlDate": self.__date,
            "jedlo_dna": "2",
            "kvalita": str(quality),
            "mnozstvo": str(quantity)
        }

        response = edupage.session.post(request_url, data=data)
        parsed_response = json.loads(response.content.decode())

        error = parsed_response.get("error")
        if error is None or error != "":
            raise FailedToRateException()


class Menu:
    def __init__(self, name: str, allergens: str, weight: str, number: str, rating: Optional[Rating]):
        self.name = name
        self.allergens = allergens
        self.weight = weight
        self.number = number
        self.rating = rating


class Lunch:
    def __init__(self, served_from: Optional[datetime], served_to: Optional[datetime],
                 amount_of_foods: int, chooseable_menus: list[str],
                 can_be_changed_until: datetime, title: str,
                 menus: List[Menu], date: datetime,
                 boarder_id: str):
        self.date = date
        self.served_from = served_from
        self.served_to = served_to
        self.amount_of_foods = amount_of_foods
        self.chooseable_menus = chooseable_menus
        self.can_be_changed_until = can_be_changed_until
        self.title = title
        self.menus = menus
        self.__boarder_id = boarder_id

    def __iter__(self):
        return iter(self.menus)

    def __make_choice(self, edupage: EdupageModule, choice_str: str):
        request_url = f"https://{edupage.subdomain}.edupage.org/menu/"

        boarder_menu = {
            "stravnikid": self.__boarder_id,
            "mysqlDate": self.date.strftime("%Y-%m-%d"),
            "jids": {
                "2": choice_str
            },
            "view": "pc_listok",
            "pravo": "Student"
        }

        data = {
            "akcia": "ulozJedlaStravnika",
            "jedlaStravnika": json.dumps(boarder_menu)
        }

        response = edupage.session.post(request_url, data=data).content.decode()

        if json.loads(response).get("error") != "":
            raise FailedToChangeLunchError()

    def choose(self, edupage: EdupageModule, number: int):
        letters = "ABCDEFGH"
        letter = letters[number - 1]

        self.__make_choice(edupage, letter)

    def sign_off(self, edupage: EdupageModule):
        self.__make_choice(edupage, "AX")


class Lunches(Module):
    @ModuleHelper.logged_in
    def get_lunch(self, date: datetime):
        date_strftime = date.strftime("%Y%m%d")
        request_url = f"https://{self.edupage.subdomain}.edupage.org/menu/?date={date_strftime}"
        response = self.edupage.session.get(request_url).content.decode()

        boarder_id = response.split("var stravnikid = \"")[1].split("\"")[0]
        lunch_data = json.loads(response.split("var novyListok = ")[1].split(";")[0])

        lunch = lunch_data.get(date.strftime("%Y-%m-%d"))

        if lunch is None:
            return None

        lunch = lunch.get("2")

        served_from_str = lunch.get("vydaj_od")
        served_to_str = lunch.get("vydaj_do")

        if served_from_str:
            served_from = datetime.strptime(served_from_str, "%H:%M")
        else:
            served_from = None

        if served_to_str:
            served_to = datetime.strptime(served_to_str, "%H:%M")
        else:
            served_to = None

        title = lunch.get("nazov")

        amount_of_foods = lunch.get("druhov_jedal")
        chooseable_menus = list(lunch.get("choosableMenus").keys())

        can_be_changed_until = lunch.get("zmen_do")

        menus = []

        for food in lunch.get("rows"):
            if not food:
                continue

            name = food.get("nazov")
            allergens = food.get("alergenyStr")
            weight = food.get("hmotnostiStr")
            number = food.get("menuStr")
            rating = None

            if number is not None:
                number = number.replace(": ", "")
                rating = lunch.get("hodnotenia")
                if rating is not None and rating:
                    rating = rating.get(number)

                    [quality, quantity] = rating

                    quality_average = quality.get("priemer")
                    quality_ratings = quality.get("pocet")

                    quantity_average = quantity.get("priemer")
                    quantity_ratings = quantity.get("pocet")

                    rating = Rating(date.strftime("%Y-%m-%d"), boarder_id,
                                    quality_average, quantity_average,
                                    quality_ratings, quantity_ratings)
                else:
                    rating = None
            menus.append(Menu(name, allergens, weight, number, rating))

        return Lunch(served_from, served_to, amount_of_foods,
                     chooseable_menus, can_be_changed_until,
                     title, menus, date, boarder_id)
